from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User, Permission
from django.template.defaultfilters import slugify
from django.core.urlresolvers import reverse
from pgauth.base import LocalRoleMixin, Role
from pgauth.models import UserGroup
from pgauth.settings import VIEWER_ROLE_ID
from sharing import SharingMixin
from relatable import RelatableMixin


class BaseContent(models.Model, LocalRoleMixin, SharingMixin, RelatableMixin):

    """ All Djinn content extends this base class. Or should... """

    title = models.CharField(_('Title'), max_length=200)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)
    creator = models.ForeignKey(User, related_name='%(class)s_creator')
    changed_by = models.ForeignKey(User, related_name='%(class)s_changed_by')
    removed_creator_name = models.CharField(_('Creator naam'), max_length=100,
                                            blank=True, null=True)
    userkeywords = models.CharField(_('Keywords'), max_length=500,
                                    null=True, blank=True)
    parentusergroup = models.ForeignKey(
        UserGroup,
        related_name='%(class)s_parentusergroup',
        null=True, blank=True, on_delete=models.SET_NULL)
    is_tmp = models.BooleanField(_('Temporary'), default=False)

    # Create a temporary object during add?
    #
    create_tmp_object = False

    def get_cache_key(self):

        return "%s_%s_%s" % (self.app_label, self.ct_name, self.id)

    def save(self, *args, **kwargs):

        if self.userkeywords:
            self.userkeywords = self.userkeywords.replace("'", '')
            self.userkeywords = " ".join(self.userkeywords.split()[:10])
        super(BaseContent, self).save(*args, **kwargs)

    def __unicode__(self):

        return self.title

    @property
    def slug(self):

        return slugify(self.title)

    @property
    def ct_class(self):

        """ Use this method at your own peril... it's expensive"""

        return ContentType.objects.get_for_model(self.__class__)

    @property
    def ct_name(self):
        return self.__class__.__name__.lower()

    @property
    def app_label(self):

        try:
            return self._meta.app_label
        except:
            return self.__module__.split(".")[0]

    @property
    def ct_label(self):

        """ Display name """

        return _(self.__class__.__name__)

    @property
    def in_closed_group(self):

        return self.parentusergroup and self.parentusergroup.profile and \
            self.parentusergroup.profile.is_closed

    @property
    def is_public(self):

        """ We are public if we are not in a closed group, and if we
        are published"""

        if self.in_closed_group:
            return False

        if not self.is_published():
            return False

        return True

    @property
    def permission_authority(self):

        """ Return the permission authority for this object. This is
        usually the object itself, but in some cases, for example
        child objects like user profile phonenumbers, the check may be
        deferred to the parent object """

        return self

    def is_published(self):
        '''
        By default everything is published
        (known to be overriden by PublishableBaseContent)
        '''
        return True

    @property
    def keywordslist(self):
        if self.userkeywords:
            return self.userkeywords.split()
        return []

    def pre_delete(self):

        """ May raise DeleteError """

        pass

    def delete(self):

        for lrole in self.get_local_roles():
            lrole.delete()

        return super(BaseContent, self).delete()

    def get_absolute_url(self):

        return reverse('%s_view_%s' % (self.app_label, self.ct_name),
                       kwargs={"slug": self.slug, "pk": str(self.id)})

    def get_local_roles(self, **kwargs):

        """ Override get_local_roles, so as to be able to add viewers """

        roles = super(BaseContent, self).get_local_roles(**kwargs)

        if kwargs.get("as_role", False):
            if self.is_public:

                viewer = Role.objects.filter(
                    name=VIEWER_ROLE_ID).select_related()
                roles = roles | viewer

            elif self.in_closed_group and kwargs.get("user", None):

                if kwargs['user'] in self.parentusergroup.members.all():

                    viewer = Role.objects.filter(
                        name=VIEWER_ROLE_ID).select_related()

                    roles = roles | viewer

        return roles

    @property
    def viewers(self):

        """ Return a list of all unique users and groups that can
        'view' this content."""

        _viewers = set()

        view = Permission.objects.get(codename="view")

        for lrole in self.get_local_roles():
            if view in lrole.role.permissions.all():

                try:
                    viewuser = lrole.user
                except User.DoesNotExist:
                    viewuser = None

                if viewuser:
                    _viewers.add("user_%s" % viewuser.username)
                else:
                    try:
                        viewusergroup = lrole.usergroup
                    except UserGroup.DoesNotExist:
                        viewusergroup = None

                    if viewusergroup:
                        _viewers.add("group_%d" % viewusergroup.id)

        if self.parentusergroup and not self.ct_name == "groupprofile":
            if self.parentusergroup.membership_type != 1:
                _viewers.add("group_users")
            else:
                _viewers.add("group_%d" % self.parentusergroup.id)
        elif self.parentusergroup and self.ct_name == "groupprofile":
            _viewers.add("group_users")
        elif not self.parentusergroup:
            _viewers.add("group_users")

        return list(_viewers)

    class Meta:
        abstract = True


class FKContentMixin(object):

    """ Marker class for content that has a foreign key to some parent """



from datetime import datetime
from django.db import models
from django.utils.translation import ugettext_lazy as _
from base import BaseContent


class PublishableContent(BaseContent):

    """ Anything that is publishable should extend this class """

    publish_from = models.DateTimeField(_('Publish from'), null=True,
                                        db_index=True,
                                        blank=True, default=None)
    publish_to = models.DateTimeField(_('Publish to'), null=True,
                                      db_index=True,
                                      blank=True, default=None)
    publish_notified = models.BooleanField(
        _('Event sent'), default=False,
        help_text=_("Publish event is sent"))

    def is_published(self):

        """ Are we published? This is true iff:
          - not initial still
          - publish from is set and is earlier than now
          - if publish to is set, it is later than now
        """

        now = datetime.now()

        return not self.is_tmp and \
            (self.publish_from and self.publish_from <= now) and \
            (not self.publish_to or self.publish_to > now)

    def is_scheduled(self):

        """
        Are we scheduled to be published later?
        """

        now = datetime.now()
        return not self.is_tmp and \
            (self.publish_from and self.publish_from > now) and \
            (not self.publish_to or self.publish_to > now)

    @property
    def acquire_global_roles(self):

        """ Do we need to acquire global roles? Only if published... """

        if not self.is_published():
            return False
        else:
            return True

    class Meta:
        abstract = True



import logging
from importlib import import_module
from django.db import models
from django.db.models import get_model
from django.conf.urls.defaults import url, patterns
from djinn_core.utils import extends
from djinn_contenttypes.models.base import FKContentMixin
from base import DetailView, CreateView, UpdateView, DeleteView


LOGGER = logging.getLogger("djinn_contenttypes")


def generate_model_urls(*models, **kwargs):

    """ Generate the urls for a view, given the conventions used by
    Djinn.  To be able to find specific views and forms, these need to
    be imported.  Best do this in urls.py where you'll most likely
    call this function to generate your urls.  You may provide 'name'
    in kwargs, to override the naming convention for view names
    <modulename>_<mode>_<model>. This is handy for dynamic models, for
    instance djinn_profiles.UserProfile. Provide name as a tuple of
    (<module prefix>, <model>), i.e. name=("djinn_profiles", "userprofile")
    """

    views = []
    name = kwargs.get("name", None)

    for model in models:

        modelname = model.__name__.lower()
        modulename = model.__module__.split(".")[0]

        if kwargs.get("view", True):
            if name:
                viewname = "%s_view_%s" % name
            else:
                viewname = None
            views.append(generate_view_url(model, modelname, modulename,
                                           name=viewname))
        if kwargs.get("edit", True):
            if name:
                editname = "%s_edit_%s" % name
            else:
                editname = None
            views.append(generate_edit_url(model, modelname, modulename,
                                           name=editname))
        if kwargs.get("add", True):
            if name:
                addname = "%s_add_%s" % name
            else:
                addname = None
            views.append(generate_add_url(model, modelname, modulename,
                                          name=addname))
        if kwargs.get("delete", True):
            if name:
                delname = "%s_delete_%s" % name
            else:
                delname = None
            views.append(generate_delete_url(model, modelname, modulename,
                                             name=delname))

    return patterns("", *views)


def generate_view_url(model, modelname, modulename, name=None):

    if model._meta.swapped:
        real_model = get_model(*model._meta.swapped.split("."))
    else:
        real_model = model

    if hasattr(real_model, "slug"):
        expr = r"^%s/(?P<pk>[\d]+)/(?P<slug>[^\/]+)/?$" % modelname
    else:
        expr = r"^%s/(?P<pk>[\d]+)/?$" % modelname

    name = name or "%s_view_%s" % (modulename, modelname)
    view = find_view(model, modulename)

    return url(expr, view, name=name)


def generate_delete_url(model, modelname, modulename, name=None):

    expr = r"^delete/%s/(?P<pk>[\d]+)/?$" % modelname
    name = name or "%s_delete_%s" % (modulename, modelname)
    view = find_view(model, modulename, suffix="DeleteView",
                     default=DeleteView)

    return url(expr, view, name=name)


def generate_add_url(model, modelname, modulename, name=None):

    expr = r"^add/%s/?$" % modelname
    name = name or "%s_add_%s" % (modulename, modelname)
    form_class = find_form_class(model, modulename)

    kwargs = {}

    if form_class:
        kwargs["form_class"] = form_class

    if extends(model, FKContentMixin):

        kwargs['fk_fields'] = [f.name for f in model._meta.fields if
                               isinstance(f, models.ForeignKey)]
        expr = r"^add/%s" % modelname

        for fk_field in kwargs['fk_fields']:
            expr += "/(?P<%s>.+)" % fk_field
        expr += "/?$"

    view = find_view(model, modulename, suffix="CreateView",
                     default=CreateView,
                     **kwargs)

    return url(expr, view, name=name)


def generate_edit_url(model, modelname, modulename, name=None):

    """ Generate edit URL. Override name by providing it in the
    keyword arguments """

    expr = r"^edit/%s/(?P<pk>[\d]+)/?$" % modelname
    name = name or "%s_edit_%s" % (modulename, modelname)
    form_class = find_form_class(model, modulename)

    kwargs = {}

    if form_class:
        kwargs["form_class"] = form_class

    view = find_view(model, modulename, suffix="UpdateView",
                     default=UpdateView,
                     **kwargs)

    return url(expr, view, name=name)


def find_view(model, modulename, suffix="View", default=DetailView, **kwargs):

    """ Try to find the view for the model. This checks the views.py
    of the model's module, and also views.<modelname>.py """

    viewclassname = "%s%s" % (model.__name__, suffix)
    modelname = model.__name__.lower()
    view_class = module = None

    if model._meta.swapped:
        modulename, modelname = model._meta.swapped.lower().split(".")

    try:
        module = import_module("%s.views.%s" % (modulename, modelname))
    except ImportError:
        try:
            module = import_module("%s.views" % modulename)
        except ImportError:
            LOGGER.info("No views module found for %s" % modulename)

    if module:
        try:
            view_class = getattr(module, viewclassname)
        except AttributeError:
            LOGGER.info("No view found for %s" % modelname)

    if not view_class:
        view_class = default
        kwargs.update({"model": model})

    return view_class.as_view(**kwargs)


def find_form_class(model, modulename):

    """ Try to find form class either in forms.py or in a separate file
    within forms named after the lower case model class. """

    formclassname = "%sForm" % model.__name__
    modelname = model.__name__.lower()
    form_class = module = None

    if model._meta.swapped:
        modulename, modelname = model._meta.swapped.split(".")
        modelname = modelname.lower()

    try:
        module = import_module("%s.forms.%s" % (modulename, modelname))
    except ImportError:
        try:
            module = import_module("%s.forms" % modulename)
        except ImportError:
            LOGGER.info("No forms module found for %s" % modulename)

    if module:
        try:
            form_class = getattr(module, formclassname)
        except AttributeError:
            LOGGER.info("No form found for %s" % modelname)

    return form_class



from django.views.generic.edit import FormView
from django.utils.translation import ugettext_lazy as _
from djinn_contenttypes.views.base import CTMixin, MimeTypeMixin
from djinn_contenttypes.forms.share import ShareForm
from pgtimeline.models import QueuedActivity
from pgevents.events import Events
from pgevents.settings import RETWEET_CONTENTITEM, RETWEET_ACTIVITY


class ShareView(MimeTypeMixin, FormView, CTMixin):

    """Share the given content with other users or groups. This will
    result in a notification to the users/groups. The share form is
    shown in a modal window.
    """

    template_name = "djinn_contenttypes/snippets/share.html"
    form_class = ShareForm
    success_url = "#"
    notification_type = RETWEET_CONTENTITEM
    mimetype = "text/plain"

    @property
    def page_title(self):

        # Translators: djinn_contenttypes share content form title
        return _("share %(obj)s with:") % {'obj': self.obj}

    @property
    def share_url(self):

        return self.request.path

    @property
    def recipient_value(self):

        form = self.get_form(self.form_class)
        return form.data.get('recipient')

    def render_recipients(self):

        """ Render the separate radio options of the recipient field """

        form = self.get_form(self.form_class)
        recipient = form.fields['recipient']
        value = form.data.get('recipient')

        options = []

        for opt in recipient.widget.get_renderer('recipient', ''):

            opt.value = value
            options.append(opt)

        return [option.render(value=value) for option in options]

    @property
    def obj(self):

        """ Accessor for object, keeping the object in a variable """

        if not hasattr(self, "_obj"):
            self._obj = self.get_object()

        return self._obj

    def form_invalid(self, form):

        return self.render_to_response(
            self.get_context_data(form=form),
            status=202
        )

    def form_valid(self, form):

        if form.cleaned_data['recipient'] == "followers":

            # Retweet must be 'sent' to all users that sender is following
            #
            Events.send(
                self.notification_type,
                user=self.request.user,
                content_item=self.get_object(),
                message=form.cleaned_data['message'])

        elif form.cleaned_data['recipient'] == "group":

            for group in form.cleaned_data['group']:

                Events.send(
                    self.notification_type,
                    user=self.request.user,
                    to_usergroup=group.usergroup,
                    content_item=self.get_object(),
                    message=form.cleaned_data['message'])

        elif form.cleaned_data['recipient'] == "user":

            for user in form.cleaned_data['user']:

                Events.send(
                    self.notification_type,
                    user=self.request.user,
                    to_user=user.user,
                    content_item=self.get_object(),
                    message=form.cleaned_data['message'])

        return super(ShareView, self).form_valid(form)


class ShareActivityView(ShareView):

    notification_type = RETWEET_ACTIVITY

    @property
    def page_title(self):

        # Translators: djinn_contenttypes share message form title
        return _("Share this message with:")

    def get_object(self, queryset=None):

        return QueuedActivity.objects.get(pk=self.kwargs['activity_id'])



from django import forms
from django.utils.translation import ugettext_lazy as _
from djinn_forms.widgets.share import ShareWidget


RECIPIENT_CHOICES = (
    ("followers", _("Colleagues that follow you")),
    ("group", _("Members of one or more groups")),
    ("user", _("One or more colleagues"))
)


class ShareField(forms.Field):

    def prepare_value(self, data):

        return data or {'add': [], 'rm': []}

    def clean(self, value):

        return value.get("add")


class ShareForm(forms.Form):

    # Translators: Content share form help
    help = _("Share this content")

    recipient = forms.CharField(
        label=_("Share with"), required=True,
        widget=forms.RadioSelect(choices=RECIPIENT_CHOICES)
    )

    group = ShareField(
        # Translators: share form group label
        label=_("Share with group"),
        required=False,
        widget=ShareWidget(
            attrs={'searchfield': 'title_auto',
                   # Translators: share user placeholder
                   'hint': _("Find group to share with"),
                   'content_type': ["djinn_profiles.groupprofile"]}
        )
    )

    user = ShareField(
        # Translators: share form user label
        label=_("Share with user"),
        required=False,
        widget=ShareWidget(
            attrs={'searchfield': 'title_auto',
                   # Translators: share user placeholder
                   'hint': _("Find user to share with"),
                   'content_type': ["djinn_profiles.userprofile"]}
        )
    )

    message = forms.CharField(
        # Translators: share form message label
        label=_("Message to send with share"),
        max_length=200,
        help_text="Maximaal 200 karakters",
        widget=forms.Textarea(
            attrs={"class": "expand count_characters init_expansion",
                   'data-maxchars': '200',
                   "cols": 100}),
        required=True
    )

    def clean(self):

        cleaned_data = super(ShareForm, self).clean()

        if cleaned_data.get("recipient", "") == "group" and \
           not cleaned_data.get("group"):

            # Translators: sharing group required
            msg = _("Selecting a group is required")

            self._errors["group"] = self.error_class([msg])

        if cleaned_data.get("recipient", "") == "user" and \
           not cleaned_data.get("user"):

            # Translators: sharing user required
            msg = _("Selecting a user is required")

            self._errors["user"] = self.error_class([msg])

        return cleaned_data

    



from django.views.generic.detail import DetailView as BaseDetailView
from django.views.generic.edit import UpdateView as BaseUpdateView
from django.views.generic.edit import DeleteView as BaseDeleteView
from django.views.generic.edit import CreateView as BaseCreateView
from django.views.generic.base import TemplateResponseMixin
from django.http import HttpResponseRedirect, HttpResponse, \
    HttpResponseForbidden
from django.db.models import get_model
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson as json
from djinn_core.utils import implements
from djinn_contenttypes.registry import CTRegistry
from djinn_contenttypes.utils import get_object_by_ctype_id, has_permission
from djinn_contenttypes.models.base import BaseContent
from djinn_contenttypes.utils import json_serializer
from pgauth.models import UserGroup
from django.core.exceptions import ImproperlyConfigured


class TemplateResolverMixin(object):

    @property
    def app_label(self):
        return self.model.__module__.split(".")[0]

    @property
    def ct_name(self):
        return self.model.__name__.lower()

    @property
    def ct_label(self):

        return _(self.model.__name__)

    def get_template_names(self):

        if self.template_name:
            return [self.template_name]

        if self.request.GET.get("modal", False) or self.request.is_ajax():
            modal = "_modal"
        else:
            modal = ""

        return ["%s/%s_%s%s.html" % (self.app_label, self.ct_name, self.mode,
                modal),
                "djinn_contenttypes/base_%s%s.html" % (self.mode, modal)
                ]

    @property
    def delete_url(self):

        return reverse("%s_delete_%s" % (self.app_label, self.ct_name),
                       kwargs={'pk': self.object.id})

    @property
    def add_url(self):

        return reverse("%s_add_%s" % (self.app_label, self.ct_name),
                       kwargs=self.kwargs)

    @property
    def edit_url(self):

        return reverse("%s_edit_%s" % (self.app_label, self.ct_name),
                       kwargs={'pk': self.object.id})

    @property
    def view_url(self):

        kwargs = {"pk": self.object.id}

        if hasattr(self.object, "slug"):
            kwargs.update({"slug": self.object.slug})

        return reverse('%s_view_%s' % (self.app_label, self.ct_name),
                       kwargs=kwargs)


class MimeTypeMixin(object):

    """ Mixin class to set mimetype. Make sure this is the first class in the
    line of extended classes that do render_to_response... """

    mimetype = "text/html"

    def render_to_response(self, context, **response_kwargs):

        """ Override so as to add mimetype """

        response_kwargs['mimetype'] = self.mimetype

        return super(MimeTypeMixin, self).render_to_response(
            context, **response_kwargs)


class CTMixin(object):

    """ Detailview that applies to any content, determined by the url parts """

    def get_object(self, queryset=None):

        """ Retrieve context from URL parts app, ctype and id."""

        return get_object_by_ctype_id(
            self.kwargs['ctype'],
            self.kwargs.get('id', self.kwargs.get('pk', None)))


class SwappableMixin(object):

    """ Allow for swapped models. This is an undocumented feature of
    the meta class for now, that enables user defined overrides for
    models (i.e. contrib.auth.User)"""

    @property
    def real_model(self):

        if not self.model._meta.swapped:
            return self.model
        else:
            module, model = self.model._meta.swapped.split(".")
            return get_model(module, model)

    def get_queryset(self):

        """ Override default get_queryset to allow for swappable models """

        if self.queryset is None:
            if self.model:
                return self.real_model._default_manager.all()
            else:
                raise ImproperlyConfigured(
                    "%(cls)s is missing a queryset. Define "
                    "%(cls)s.model, %(cls)s.queryset, or override "
                    "%(cls)s.get_queryset()." % {
                        'cls': self.__class__.__name__
                    })
        return self.queryset._clone()


class AcceptMixin(object):

    """ Use the accept header to determine the response. Supported are
    text/html, text/plain and application/json. Mixin class should come as
    first extended class... """

    @property
    def is_json(self):

        return "application/json" in self.request.META.get("HTTP_ACCEPT", [])

    @property
    def is_text(self):

        return "text/plain" in self.request.META.get("HTTP_ACCEPT", [])

    def render_to_response(self, context, **response_kwargs):

        if self.is_json:

            try:
                del response_kwargs['mimetype']
            except:
                pass

            response_kwargs["content_type"] = 'application/json'

            return HttpResponse(
                json.dumps(context, skipkeys=True,
                           default=json_serializer),
                **response_kwargs)
        elif self.is_text:
            try:
                del response_kwargs['mimetype']
            except:
                pass

            response_kwargs["content_type"] = 'text/plain'
            return TemplateResponseMixin.render_to_response(self,
                                                            context,
                                                            **response_kwargs)
        else:
            return TemplateResponseMixin.render_to_response(self,
                                                            context,
                                                            **response_kwargs)


class AbstractBaseView(TemplateResolverMixin, SwappableMixin, AcceptMixin):

    def _determine_content_type(self, default="text/html"):

        content_type = default

        if self.request.is_ajax():
            content_type = "text/plain"

        if "application/json" in self.request.META.get("HTTP_ACCEPT", []):
            content_type = "application_json"

        return content_type


class DetailView(AbstractBaseView, BaseDetailView):

    """ Detail view for simple content, not related, etc. All intranet
    detail views should extend this view.
    """

    mode = "detail"

    def get_template_names(self):

        """ If the request is Ajax, and no one asked for a modal view,
        assume that we need to return a record like view"""

        if self.template_name:
            return [self.template_name]

        templates = super(DetailView, self).get_template_names()

        if self.request.is_ajax() and \
                not self.request.REQUEST.get("modal", False):
            templates = ["%s/snippets/%s.html" %
                         (self.app_label, self.ct_name)] \
                + templates

        return templates

    def get(self, request, *args, **kwargs):

        """ We need our own implementation of GET, to be able to do permission
        checks on the object """

        self.object = self.get_object()

        perm = CTRegistry.get(self.ct_name).get("view_permission",
                                                'contenttypes.view')

        if not has_permission(perm, self.request.user, self.object):
            return HttpResponseForbidden()

        context = self.get_context_data(object=self.object)
        content_type = self._determine_content_type()

        return self.render_to_response(context, content_type=content_type)


class CTDetailView(CTMixin, DetailView):

    """Detailview that applies to any content, determined by the url
    parts. This means that the model is really variable! """

    @property
    def model(self):

        return self.get_object().__class__


class CreateView(TemplateResolverMixin, SwappableMixin, AcceptMixin,
                 BaseCreateView):

    mode = "add"
    fk_fields = []

    def get_initial(self):

        initial = {}

        for fld in self.request.GET.keys():
            initial[fld] = self.request.GET[fld]

        for fld in self.fk_fields:
            initial[fld] = self.kwargs[fld]

        return initial

    def get_form_kwargs(self):

        """ Add user to kwargs for form. """

        kwargs = super(CreateView, self).get_form_kwargs()

        if getattr(self.form_class, "user_support", False):
            kwargs['user'] = self.request.user

        return kwargs

    def get_success_url(self):

        return self.view_url

    def get_object(self, queryset=None):

        if not getattr(self.real_model, "create_tmp_object", False):
            return None
        else:
            if self.request.REQUEST.get("tmp_id"):
                obj = self.real_model.objects.get(
                    id=self.request.REQUEST.get("tmp_id"))
            else:
                obj = self.real_model.objects.create(
                    creator=self.request.user,
                    is_tmp=True,
                    changed_by=self.request.user
                    )

                # Set any data that is available, i.e. through initial data
                #
                if self.get_initial():
                    form_class = self.get_form_class()

                    kwargs = self.get_form_kwargs()
                    kwargs['data'] = self.get_initial()

                    form = form_class(**kwargs)
                    form.cleaned_data = {}

                    for field in self.get_initial().keys():
                        value = form.fields[field].clean(form.data[field])
                        setattr(obj, field, value)

            obj.save()

            return obj

    def get(self, request, *args, **kwargs):

        """ Override get so as to be able to check permissions """

        # Set object. If not set, odd behaviour of the django generic
        # view classes may occur...
        self.object = None

        # TODO: this should be the more specific permission
        perm = CTRegistry.get(self.ct_name).get("add_permission",
                                                'contenttypes.add_contenttype')

        # TODO: check if this still exists...
        ugid = kwargs.get('parentusergroup', None)

        if ugid:
            context = UserGroup.objects.get(id=int(ugid)).profile
        else:
            context = None

        if not self.request.user.has_perm(perm, obj=context):
            return HttpResponseForbidden()

        self.object = self.get_object()

        form_class = self.get_form_class()
        form = self.get_form(form_class)

        return self.render_to_response(self.get_context_data(form=form))

    def post(self, request, *args, **kwargs):

        # Set object. If not set, odd behaviour of the django generic
        # view classes may occur...
        self.object = None

        perm = CTRegistry.get(self.ct_name).get("add_permission",
                                                'contenttypes.add_contenttype')

        pugid = kwargs.get('parentusergroup', None)

        if pugid:
            theobj = UserGroup.objects.get(id=int(pugid)).profile
        else:
            theobj = None
        if not self.request.user.has_perm(perm, obj=theobj):
            return HttpResponseForbidden()

        self.object = self.get_object()

        if self.request.POST.get('action', None) == "cancel":

            return self.handle_cancel()
        else:
            form_class = self.get_form_class()
            form = self.get_form(form_class)

            if form.is_valid():
                return self.form_valid(form)
            else:
                return self.form_invalid(form)

    def form_valid(self, form):

        """ If the form is valid, first let the form.save create the
        object. We can't commit straight away, since essential data for
        the content type is not in the form (creator, changed_by).
        Set those, then commit. """

        self.object = form.save(commit=False)

        # Assume, but not overly, that this is BaseContent...
        #
        try:
            self.object.creator = self.request.user
            self.object.changed_by = self.request.user
            self.object.is_tmp = False
        except:
            pass

        # And now for real!
        #
        self.object = form.save()

        if implements(self.object, BaseContent):
            if not self.object.get_owner():
                self.object.set_owner(self.request.user)

        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form):
        return self.render_to_response(self.get_context_data(form=form),
                                       status=202)

    def handle_cancel(self):

        """Default action upon cancel of edit. If we have an object, and it
        is temporary, delete and return to user home. If it is not
        temporary, go to view of object. If there is no object, go home...
        """

        if self.object and getattr(self.object, "is_tmp", False):
            self.object.delete()

        return HttpResponseRedirect(
            self.request.user.profile.get_absolute_url())


class UpdateView(TemplateResolverMixin, SwappableMixin, AcceptMixin,
                 BaseUpdateView):

    mode = "edit"

    def get_success_url(self):

        return self.view_url

    @property
    def partial(self):

        """ Allow partial update """

        return self.request.REQUEST.get('partial', False)

    def get_form_kwargs(self):

        kwargs = super(UpdateView, self).get_form_kwargs()

        if getattr(self.form_class, "partial_support", False):
            kwargs['partial'] = self.partial

        if getattr(self.form_class, "user_support", False):
            kwargs['user'] = self.request.user

        return kwargs

    def get_initial(self):

        initial = {}

        for fld in self.request.GET.keys():
            initial[fld] = self.request.GET[fld]

        return initial

    def get(self, request, *args, **kwargs):

        """ Check permissions for editing """

        self.object = self.get_object()

        perm = CTRegistry.get(self.ct_name).get(
            "edit_permission",
            'contenttypes.change_contenttype')

        if not has_permission(perm, self.request.user, self.object):
            return HttpResponseForbidden()

        return super(UpdateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):

        """ Check whether the user wants to cancel the whole
        operation. If so, we need to check whether is was an edit, or
        initial object. If the latter is the case, delete the 'tmp'
        object."""

        self.object = self.get_object()

        perm = CTRegistry.get(self.ct_name).get(
            "edit_permission",
            'contenttypes.change_contenttype')

        if not has_permission(perm, self.request.user, self.object):
            return HttpResponseForbidden()

        if self.request.POST.get('action', None) == "cancel":

            return self.handle_cancel()
        else:
            return super(UpdateView, self).post(request, *args, **kwargs)

    def form_valid(self, form):

        if hasattr(form, "update") and self.partial:
            self.object = form.update()
        else:
            self.object = form.save()

        if implements(self.object, BaseContent):
            self.object.changed_by = self.request.user

        try:
            self.object.is_tmp = False
        except:
            pass

        self.object.save()

        if not self.partial:
            # Translators: edit form status message
            messages.success(self.request, _("Saved changes"))

        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form):
        return self.render_to_response(self.get_context_data(form=form),
                                       status=202)

    def handle_cancel(self):

        """Default action upon cancel of edit. If we have an object, and it
        is temporary, delete and return to user home. If it is not
        temporary, go to view of object. If there is no object, go home...
        """

        redir = HttpResponseRedirect(
            self.request.user.profile.get_absolute_url())

        if self.object:
            if getattr(self.object, "is_tmp", False):
                self.object.delete()
            else:
                redir = HttpResponseRedirect(self.view_url)

        return redir


class DeleteView(TemplateResolverMixin, SwappableMixin, AcceptMixin,
                 BaseDeleteView):

    mode = "delete"

    def delete(self, request, *args, **kwargs):

        self.object = self.get_object()

        perm = CTRegistry.get(self.ct_name).get(
            "delete_permission",
            'contenttypes.delete_contenttype')

        if not has_permission(perm, self.request.user, self.object):
            return HttpResponseForbidden()

        deleted = False

        try:
            # enable signal handlers to access last change info
            #
            if implements(self.object, BaseContent):
                self.object.changed_by = self.request.user

            self.object.delete()

            deleted = True
        except Exception:
            pass

        if deleted:
            if self.is_json:
                return HttpResponse(json.dumps({'message': 'bye bye'}),
                                    content_type='application/json')
            elif self.is_text:
                return HttpResponse("Bye bye", content_type='text/plain')
            else:
                return HttpResponseRedirect(self.get_success_url())
        else:
            if self.is_json:
                return HttpResponse(json.dumps({'message': 'error'}),
                                    status=202,
                                    content_type='application/json')
            elif self.is_text:
                return HttpResponse("Error", status=202,
                                    content_type='text/plain', )
            else:
                return HttpResponseRedirect(self.view_url)

    def get_success_url(self):

        """ Return the URL to which the edit/create action should
        return upon success"""

        return self.request.user.profile.get_absolute_url()

    def get(self, request, *args, **kwargs):

        """ Make sure that the confirm delete is sent as text/plain, given that
        it is show as a modal. """

        self.object = self.get_object()
        context = self.get_context_data(object=self.object, **kwargs)

        self.request.META["HTTP_ACCEPT"] = "text/plain"

        return self.render_to_response(context)



# coding=utf-8
"""
This filename should be renamed to something which is not equal to the
modulename.. it breaks the import from djinn_contenttypes
"""

from django.template import Library
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from djinn_core.utils import implements as _implements
from djinn_core.utils import object_to_urn as obj_to_urn
from djinn_core.utils import HTMLTruncate


register = Library()


@register.filter
def abbreviate(text, length=50, truncate_str='...'):
    """
    Usage: {{ sometext|abbreviate:"50" }} Abbreviates given string if
    it is more than 'length' chars, and adds trailing dots.
    """

    if len(text) > length:
        return "%s%s" % (text[:length], truncate_str)
    return text


@register.filter
def abbreviate_hellip(text, length=50):
    """
    Usage: {{ sometext|abbreviate:"50" }} Abbreviates given string if
    it is more than 'length' chars, and adds trailing horizontal ellipsis.
    """

    return abbreviate(text, length, u'…')


@register.filter
def abbreviate_html(text, length=50):

    """ Break off the text content of 'text', ignoring the html tags, and
    making sure that no tags are cut off. """

    return HTMLTruncate(length).truncate(text)


@register.inclusion_tag('djinn_contenttypes/snippets/cancel_action.html')
def cancel_action(label=None):
    return {'label': label or _("Cancel")}


@register.inclusion_tag('djinn_contenttypes/snippets/delete_action.html')
def delete_action(obj, label=None):

    """ Determine delete URL and return button """

    return {'url': reverse("%s_delete_%s" % (obj.app_label,
                                             obj.ct_name),
                           kwargs={'pk': obj.id}),
            'label': label or _("Delete")}


@register.inclusion_tag('djinn_contenttypes/snippets/edit_action.html')
def edit_action(obj, label=None):

    """ Determine edit URL and return button """

    return {'url': reverse("%s_edit_%s" % (obj.app_label,
                                           obj.ct_name),
                           kwargs={'pk': obj.id}),
            'label': label or _("Edit")}


@register.filter
def edit_permission_id(obj):

    return "%s.change_%s" % (obj.app_label, obj.ct_name)


@register.filter
def delete_permission_id(obj):

    return "%s.delete_%s" % (obj.app_label, obj.ct_name)


@register.filter
def implements(instance, clazz):

    return _implements(instance, clazz)


@register.filter
def object_to_urn(obj):
    return obj_to_urn(obj)



from django.conf.urls.defaults import patterns, url
from djinn_contenttypes.views.share import ShareView, ShareActivityView


urlpatterns = patterns(

    '',

    url(r'^content/share/(?P<ctype>[\w]+)/(?P<id>[\d]+)/?$',
        ShareView.as_view(),
        name="djinn_contenttypes_share"),

    url(r'^content/share/(?P<activity_id>[\d]+)/?$',
        ShareActivityView.as_view(),
        name="djinn_contenttypes_share")
)



import os
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from photologue.models import ImageModel
import magic


class Attachment(models.Model):

    title = models.CharField(_('title'), max_length=100, null=True, blank=True)

    @property
    def permission_authority(self):

        return None

    @property
    def _file(self):

        raise NotImplementedError

    @property
    def absolute_url(self):
        return os.path.join(settings.MEDIA_URL, self._file.name)

    @property
    def absolute_path(self):
        return self._file.path

    def mimetype(self, lexical=False):

        if lexical:
            magic.from_file(self._file.name)
        else:
            magic.from_file(self._file.name, mime=True)

    def __unicode__(self):
        if self.title:
            return u"%s" % self.title

        return u"%s" % self._file.name

    class Meta:
        abstract = True
        app_label = "djinn_contenttypes"


class ImgAttachment(Attachment, ImageModel):

    @property
    def _file(self):

        return self.image

    class Meta:
        app_label = "djinn_contenttypes"


class FileAttachment(Attachment):

    # This is not a typo.
    #
    ffile = models.FileField(upload_to='files')

    @property
    def _file(self):

        return self.ffile



from django import forms
from djinn_contenttypes.models import ImgAttachment
from djinn_contenttypes.forms.base import MetaFieldsMixin


class ImgAttachmentForm(MetaFieldsMixin, forms.ModelForm):

    class Meta:
        model = ImgAttachment
        fields = ["title"]



from django.forms import CharField
from django.template.defaultfilters import removetags


class NoScriptCharField(CharField):

    """
    Specific CharField that filters out <script>-tags.
    """

    def to_python(self, value):

        """
        Strips <script>-tags from the field.
        """

        if value:
            value = value.replace("\r\n", "\n")
            return removetags(value, 'script')
        else:
            return None


from django.db import models
from django.utils.translation import ugettext_lazy as _
from djinn_contenttypes.utils import get_comment_model


class Commentable(models.Model):

    """
    Contains methods that apply to content for which you can add
    'Reacties' Comments do not use the SimpleRelation type, but are
    connected to content using CommentRelation
    """

    comments_enabled = models.BooleanField(_("Collega's kunnen reageren"),
                                           default=1)

    @property
    def comment_model(self):

        return get_comment_model()

    def get_comments(self):
        '''
        Returns all comments related to this content_item instance,
        regardless of public true/false and ownership of comment
        '''
        return self.comment_model.objects.get_for_contentitem(self)

    def add_comment(self, comment):

        """ Add comment to the content. """
        comment.add_relation_to_instance(self)

    class Meta:
        abstract = True



from datetime import datetime
from django import forms
from django.utils.translation import ugettext_lazy as _
from djinn_forms.fields.share import ShareField
from djinn_forms.forms.share import ShareMixin
from pgauth.models import UserGroup
from pgauth.settings import OWNER_ROLE_ID, EDITOR_ROLE_ID
from djinn_forms.fields.role import LocalRoleSingleField
from djinn_forms.fields.relate import RelateField
from djinn_forms.fields.keyword import KeywordField
from djinn_forms.widgets.relate import RelateSingleWidget, RelateWidget
from djinn_forms.forms.relate import RelateMixin
from djinn_forms.widgets.datetimewidget import DateTimeWidget


class PartialUpdateMixin(object):

    partial_support = True

    def update(self, commit=True):

        """ Allow for updates of only the fields available in the form """

        for f in self.instance._meta.fields:
            if f.attname in self.fields:
                setattr(self.instance, f.attname,
                        self.cleaned_data[f.attname])
            if commit:
                try:
                    self.instance.save()
                except:
                    return False
            return self.instance


class MetaFieldsMixin(object):

    """ This mixin actually honours the fields setting of the meta info """

    def __iter__(self):

        for name in [name for name in self.fields if
                     name in self._meta.fields]:
            yield self[name]


class BaseForm(PartialUpdateMixin, forms.ModelForm):

    user_support = True

    def __init__(self, *args, **kwargs):

        try:
            partial = kwargs.pop('partial')
        except:
            partial = False

        try:
            user = kwargs.pop("user")
        except:
            user = None

        super(BaseForm, self).__init__(*args, **kwargs)

        if partial:
            self.fields = dict((fname, field) for fname, field in
                               self.fields.items() if
                               fname in self.data.keys())

        self.user = user

    class Meta:
        exclude = ["creator", "changed_by"]


class BaseSharingForm(BaseForm, RelateMixin, ShareMixin):

    owner = LocalRoleSingleField(
        OWNER_ROLE_ID,
        ["pgprofile.userprofile"],
        # Translators: Contentype owner label
        label=_("Owner"),
        required=False,
        widget=RelateSingleWidget(
            attrs={'searchfield': 'title_auto',
                   #Translators: content type owner hint
                   'hint': _("Select a name")
                   })
        )

    shares = ShareField(
        EDITOR_ROLE_ID,
        ["pgprofile.userprofile", "pgprofile.groupprofile"],
        # Translators: Contentype shares/editors label
        label=_("Editors"),
        # Translators: content shares help
        help_text=_("Select users or groups to share editing role"),
        required=False,
        widget=RelateWidget(
            attrs={'searchfield': 'title_auto',
                   #Translators: content type owner hint
                   'hint': _("Select a user or group name ")
                   })
        )


class BaseContentForm(BaseSharingForm):

    # Translators: contenttypes title label
    title = forms.CharField(label=_("Title"),
                            max_length=255,
                            widget=forms.TextInput())

    # Translators: contenttypes usergroup label
    parentusergroup = forms.ModelChoiceField(label=_("Add to group"),
                                             required=False,
                                             queryset=UserGroup.objects.all())

    publish_from = forms.DateTimeField(
        # Translators: contenttypes publish_from label
        label=_("Publish from"),
        # Translators: contenttypes publish_from help
        help_text=_("Enter a publish-from date and time"),
        required=False,
        widget=DateTimeWidget(
            attrs={'date_hint': _("Date"),
                   'time_hint': _("Time")}
            )
        )

    publish_to = forms.DateTimeField(
        # Translators: contenttypes publish_to label
        label=_("Publish to"),
        # Translators: contenttypes publish_to help
        help_text=_("Enter a publish-to date and time"),
        required=False,
        widget=DateTimeWidget(
            attrs={'date_hint': _("Date"),
                   'time_hint': _("Time")}
            )
        )

    userkeywords = KeywordField(
        # Translators: contenttypes userkeywords label
        label=_("Keywords"),
        required=False,
        # Translators: contenttypes userkeywords help
        help_text=_("Enter keywords separated by spaces"),
    )

    related = RelateField(
        "related_content",
        [],
        # Translators: contenttypes related label
        label=_("Related content"),
        required=False,
        # Translators: Translators: contenttypes related hint
        help_text=_("Select related content"),
        widget=RelateWidget(
            attrs={'hint': _("Search relation"),
                   'searchfield': 'title_auto',
                   'template_name':
                   'djinn_forms/snippets/relatesearchwidget.html',
                   'search_url': '/content_search/', },
            )
        )

    def __init__(self, *args, **kwargs):

        super(BaseContentForm, self).__init__(*args, **kwargs)

        self.init_relation_fields()
        self.init_share_fields()

        self.fields['parentusergroup'].choices = self._group_choices()
        self.fields['userkeywords'].show_label = True

        # If there is no parentusergroup in the instance, and the instance is
        # a temporary one, set the group to -1.
        #
        if not self.instance.parentusergroup and self.instance.is_tmp:
            self.initial['parentusergroup'] = -1

    def _group_choices(self):

        """Populate group selector. This adds the special case '-1' for no
        selection made, so we can make sure the user needs to either
        select a group, or select 'no group'.
        """

        if self.user and self.user.is_superuser:
            groups = UserGroup.objects.all()
        elif self.user:
            groups = self.user.usergroup_set.all()
        else:
            groups = UserGroup.objects.none()

        # if we already have a group set, add it.
        #
        if self.instance.parentusergroup:
            groups = groups | UserGroup.objects.filter(
                pk=self.instance.parentusergroup.id)

        groups = groups.filter(is_system=False,
                               name__isnull=False).exclude(name="").distinct()

        return [
            # Translators: djinn_contenttypes group make a choice label
            ("-1", _("Make a choice")),
            # Translators: djinn_contenttypes group no group label
            (("", _("Do not add to a group")))] + \
            [(group.id, str(group)) for group in groups]

    def save(self, commit=True):

        res = super(BaseContentForm, self).save(commit=commit)

        self.save_relations(commit=commit)
        self.save_shares(commit=commit)

        return res

    def clean(self):

        # Check whether the 'save_as_concept' was used
        #
        if self.data.get("action") == "save":
            if not self.cleaned_data['publish_from']:
                self.cleaned_data['publish_from'] = datetime.now()
        elif self.data.get("action") == "save_as_concept":
            self.cleaned_data['publish_from'] = None
            self.cleaned_data['publish_to'] = None

        try:
            if self.cleaned_data['publish_to'] < \
                    self.cleaned_data['publish_from']:
                self.cleaned_data['publish_to'] = None
        except:
            pass

        return self.cleaned_data

    def clean_parentusergroup(self):

        """The parentusergroup requires some special attention: it is not
        required, but users need to make that choice explicitly. """

        group = self.cleaned_data.get('parentusergroup')

        if group == -1:
            if group == -1:
                # Translators: djinn_contenttypes parentusergroup required
                raise forms.ValidationError(_(u"Make a choice"))

        return group
